#
# The Plone Help Center container.
#
# A Help Center is pre-populated with the following folders:
#
# /faq - contains the FAQ objects
# /how-to - contains the How-tos
# /tutorial - contains the tutorials
# /reference - contains the reference manuals
# /error - contains the error references
# /link - contains the links to other documentation
# /glossary - contains the definitions
# /video - contains video files for training/instruction
# /manual - contains reference manuals
#
# The main goals of these folders are to restrict the addable types and
# provide a sensible default view out-of-the-box, like the FAQ view.
#

from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.WorkflowCore import WorkflowException
from Products.CMFCore.utils import getToolByName
from AccessControl import ClassSecurityInfo
from Products.PloneHelpCenter.config import *
from schemata import HCRootSchema

import os
from Globals import package_home

class HelpCenter(OrderedBaseFolder):
    """A simple folderish archetype"""
    schema = HCRootSchema

    content_icon = 'helpcenter_icon.gif'

    archetype_name = 'Help Center'
    meta_type = 'HelpCenter'
    filter_content_types = 1
    _at_rename_after_creation = True
    
    allowed_content_types = ( 'HelpCenterFAQFolder'
                            , 'HelpCenterHowToFolder'
                            , 'HelpCenterTutorialFolder'
                            , 'HelpCenterReferenceManualFolder'
                            , 'HelpCenterInstructionalVideoFolder'
                            , 'HelpCenterLinkFolder'
                            , 'HelpCenterErrorReferenceFolder'
                            , 'HelpCenterGlossary' )

    security = ClassSecurityInfo()

    actions = (
        {
            'id': 'view',
            'name': 'View',
            'action': 'string:${object_url}/helpcenter_view',
            'permissions': (CMFCorePermissions.View,)
        },
        {
            'id': 'local_roles',
            'name': 'Sharing',
            'action': 'string:${object_url}/folder_localrole_form',
            'permissions': (CMFCorePermissions.ManageProperties,)
        },
    )

    def initializeArchetype(self, **kwargs):
        """Pre-poulate the help center folder with its basic folders. Also
        add a standard how-to-use-this-resource help page. The body text of
        this is found in docs/PHCManual.stx.
        """
        BaseFolder.initializeArchetype(self,**kwargs)

        if 'faq' not in self.objectIds():
            self.invokeFactory('HelpCenterFAQFolder','faq')
            obj = self['faq']
            obj.setTitle(self.translate(
					msgid='phc_faq_title',
					domain='plonehelpcenter',
					default='FAQs'))
            obj.setDescription(self.translate(
					msgid='phc_faq_description',
					domain='plonehelpcenter',
					default='Frequently Asked Questions'))
            obj.reindexObject()

        if 'how-to' not in self.objectIds():
            self.invokeFactory('HelpCenterHowToFolder','how-to')
            obj = self['how-to']
            obj.setTitle(self.translate(
					msgid='phc_howto_title',
					domain='plonehelpcenter',
					default='How-tos'))
            self['how-to'].setDescription(self.translate(
					msgid='phc_howto_description',
					domain='plonehelpcenter',
					default='Step-by-step instructions.'))
            obj.reindexObject()

        if 'tutorial' not in self.objectIds():
            self.invokeFactory('HelpCenterTutorialFolder','tutorial')
            obj = self['tutorial']
            obj.setTitle(self.translate(
					msgid='phc_tutorial_title',
					domain='plonehelpcenter',
					default='Tutorials'))
            obj.setDescription(self.translate(
					msgid='phc_tutorial_description',
					domain='plonehelpcenter',
					default='Detailed tutorials.'))
            obj.reindexObject()

        if 'manual' not in self.objectIds():
            self.invokeFactory('HelpCenterReferenceManualFolder','manual')
            obj = self['manual']
            obj.setTitle(self.translate(
					msgid='phc_referencemanual_title',
					domain='plonehelpcenter',
					default='Reference Manuals'))
            obj.setDescription(self.translate(
					msgid='phc_referencemanual_description',
					domain='plonehelpcenter',
					default='Reference manuals for larger projects.'))
            obj.reindexObject()


        if 'video' not in self.objectIds():
            self.invokeFactory('HelpCenterInstructionalVideoFolder','video')
            obj = self['video']
            obj.setTitle(self.translate(
					msgid='phc_video_title',
					domain='plonehelpcenter',
					default='Videos'))
            obj.setDescription(self.translate(
					msgid='phc_video_description',
					domain='plonehelpcenter',
					default='Instructional videos.'))
            obj.reindexObject()

        if 'error' not in self.objectIds():
            self.invokeFactory('HelpCenterErrorReferenceFolder','error')
            obj = self['error']
            obj.setTitle(self.translate(
					msgid='phc_errorreference_title',
					domain='plonehelpcenter',
					default='Error References'))
            obj.setDescription(self.translate(
					msgid='phc_errorreference_description',
					domain='plonehelpcenter',
					default='Error reference section.'))
            obj.reindexObject()

        if 'link' not in self.objectIds():
            self.invokeFactory('HelpCenterLinkFolder','link')
            obj = self['link']
            obj.setTitle(self.translate(
					msgid='phc_links_title',
					domain='plonehelpcenter',
					default='Links'))
            obj.setDescription(self.translate(
					msgid='phc_links_description',
					domain='plonehelpcenter',
					default='Links section.'))
            obj.reindexObject()

        if 'glossary' not in self.objectIds():
            self.invokeFactory('HelpCenterGlossary','glossary')
            obj = self['glossary']
            obj.setTitle(self.translate(
					msgid='phc_glossary_title',
					domain='plonehelpcenter',
					default='Glossary Definitions'))
            obj.setDescription(self.translate(
					msgid='phc_glossary_description',
					domain='plonehelpcenter',
					default='Glossary of terms.'))
            obj.reindexObject()
            
        get_transaction().commit(1)

        # Add how-to-use-this-resource how-to
        manualPath = os.path.join(package_home(GLOBALS), MANUAL_PATH)
        try:
            manualFile = open(manualPath, 'r')
            manualText = manualFile.read()
            manualFile.close()
        except IOError:
            # Silently ignore if we can't open/read the file
            pass
        else:
            # XXX somehow the following code triggers an exception
            # dunno why at the moment
            return     
            self['how-to'].invokeFactory('HelpCenterHowTo', MANUAL_ID,
                                            title       =   MANUAL_TITLE,
                                            description =   MANUAL_DESCRIPTION,
                                            sections    =  [MANUAL_SECTION])
            
            manual = getattr(self['how-to'], MANUAL_ID)
            manual.setBody(manualText, mimetype = MANUAL_MIMETYPE)
            manual.reindexObject()
    
            # Publish it
            #wftool = getToolByName(self, 'portal_workflow')
            #try:
            #    wftool.doActionFor(manual, 'publish')
            #except WorkflowException:
            #    wftool.doActionFor(manual, 'submit')



    security.declareProtected(CMFCorePermissions.View, 'getNonPHCContents')
    def getNonPHCContents(self):
        """Get a list of folder objects of types not defined by the help
        center product which have been placed in this help center.
        """
    
        phcTypes = self.allowed_content_types
        
        ttool = getToolByName(self, 'portal_types')
        hctype = getattr(ttool, self.meta_type)
        currentTypes = hctype.allowed_content_types
        
        nonPHCTypes = [t for t in currentTypes if t not in phcTypes]
        
        # Need this, else we end up with all types, not no types :)
        if not nonPHCTypes:
            return []

        return self.folderlistingFolderContents(contentFilter = 
                                                {'portal_type' : nonPHCTypes})


registerType(HelpCenter, PROJECTNAME)
